// (C) 2019-2022 GoodData Corporation
import { DataAccessImpl } from "./dataAccessImpl.js";
import { DefaultDataAccessConfig } from "../dataAccessConfig.js";
import { measureLocalId, } from "@gooddata/sdk-model";
class FilteredIterator {
    indexes;
    itemProvider;
    idx = 0;
    constructor(indexes, itemProvider) {
        this.indexes = indexes;
        this.itemProvider = itemProvider;
    }
    next = () => {
        if (this.idx >= this.indexes.length) {
            return {
                done: true,
                value: undefined,
            };
        }
        const value = this.itemProvider(this.indexes[this.idx]);
        this.idx += 1;
        return {
            done: false,
            value,
        };
    };
}
class DataSeriesCollection {
    dataAccess;
    count = 0;
    fromMeasures = [];
    fromMeasuresDef = [];
    scopingAttributes = [];
    scopingAttributesDef = [];
    constructor(dataAccess) {
        this.dataAccess = dataAccess;
        const seriesDigest = this.dataAccess.getDataAccessPointers().series;
        if (!seriesDigest) {
            return;
        }
        this.count = seriesDigest.count;
        this.fromMeasures = seriesDigest.fromMeasures;
        this.fromMeasuresDef = seriesDigest.fromMeasuresDef;
        this.scopingAttributes = seriesDigest.scopingAttributes;
        this.scopingAttributesDef = seriesDigest.scopingAttributesDef;
    }
    [Symbol.iterator] = () => {
        return this.dataAccess.getDataSeriesIterator();
    };
    allForMeasure = (localIdOrMeasure) => {
        if (!this.count) {
            return [];
        }
        const localId = typeof localIdOrMeasure === "string" ? localIdOrMeasure : measureLocalId(localIdOrMeasure);
        const indexes = this.dataAccess.findDataSeriesIndexes(localId);
        if (!indexes.length) {
            return [];
        }
        return {
            [Symbol.iterator]: () => {
                return new FilteredIterator(indexes, this.dataAccess.getDataSeries);
            },
        };
    };
    firstForMeasure = (localIdOrMeasure) => {
        if (!this.count) {
            throw new Error("there are no data series");
        }
        const localId = typeof localIdOrMeasure === "string" ? localIdOrMeasure : measureLocalId(localIdOrMeasure);
        const indexes = this.dataAccess.findDataSeriesIndexes(localId);
        if (!indexes.length) {
            throw new Error(`there are no data series for measure with local id: ${localId}`);
        }
        return this.dataAccess.getDataSeries(indexes[0]);
    };
    toArray = () => {
        return Array.from(this);
    };
}
class DataSliceCollection {
    dataAccess;
    count = 0;
    descriptors = [];
    constructor(dataAccess) {
        this.dataAccess = dataAccess;
        const slicesDigest = this.dataAccess.getDataAccessPointers().slices;
        if (!slicesDigest) {
            return;
        }
        this.count = slicesDigest.count;
        this.descriptors = slicesDigest.descriptors;
    }
    [Symbol.iterator] = () => {
        return this.dataAccess.getDataSlicesIterator();
    };
    toArray = () => {
        return Array.from(this);
    };
}
class DataAccessMethods {
    dataAccess;
    seriesCollection;
    slicesCollection;
    constructor(dataView, config) {
        this.dataAccess = new DataAccessImpl(dataView, config);
        this.seriesCollection = new DataSeriesCollection(this.dataAccess);
        this.slicesCollection = new DataSliceCollection(this.dataAccess);
    }
    series() {
        return this.seriesCollection;
    }
    slices() {
        return this.slicesCollection;
    }
}
/**
 * @internal
 */
export function newDataAccessMethods(dataView, config = DefaultDataAccessConfig) {
    return new DataAccessMethods(dataView, config);
}
//# sourceMappingURL=dataAccessMethods.js.map