import React from "react";
import { DataViewFacade, GoodDataSdkError, IExportFunction } from "../base/index.js";
/**
 * Structure specifying a particular portion of data.
 * @public
 */
export type DataViewWindow = {
    /**
     * Zero-based offsets into the data.
     */
    offset: number[];
    /**
     * Size of the window to retrieve.
     */
    size: number[];
};
/**
 * @public
 */
export type WithLoadingResult = {
    /**
     * The result of a successful loading is an instance of {@link DataViewFacade}. If this property
     * is undefined, then the data is not (yet) loaded.
     */
    result: DataViewFacade | undefined;
    /**
     * The result of failed loading. If this property is undefined, then no error has occurred (yet).
     */
    error: GoodDataSdkError | undefined;
    /**
     * Indicates whether loading is in progress or not. This value will be `false` when loading finished
     * successfully or when loading has failed. Otherwise it will be set to true.
     */
    isLoading: boolean;
    /**
     * Callback to trigger load or reload of data.
     */
    reload: () => void;
};
/**
 * @public
 */
export interface IWithLoadingEvents<TProps> {
    /**
     * If specified, this function will be called in case loading runs into an error.
     *
     * @param error - an instance of error. see also GoodDataSdkError
     * @param props - props effective at the time of load
     */
    onError?: (error: GoodDataSdkError, props: TProps) => void;
    /**
     * Called when loading starts.
     *
     * @param props - props effective at the time of load
     */
    onLoadingStart?: (props: TProps) => void;
    /**
     * Called when loading finishes.
     *
     * @param result - result wrapped in data view facade
     * @param props - props effective at the time of load
     */
    onLoadingFinish?: (result: DataViewFacade, props: TProps) => void;
    /**
     * Called when loading starts and finishes, indicating the current state using the `isLoading` parameter.
     *
     * @param isLoading - true if loading, false if no longer loading
     * @param props - props effective at the time of load
     */
    onLoadingChanged?: (isLoading: boolean, props: TProps) => void;
    /**
     * Called when loading finishes and it is possible to export the underlying data. Function that does
     * the export will be provided on the callback.
     *
     * @param exportFunction - function to call if export is desired
     * @param props - props effective at the time export is ready
     */
    onExportReady?: (exportFunction: IExportFunction, props: TProps) => void;
}
/**
 * Configuration for the {@link withExecutionLoading} HOC.
 *
 * @remarks
 * All configuration parameters can be either actual parameter values or functions to obtain them
 * from the wrapped component props.
 *
 * If functions are specified, the HOC will call them with the wrapped component props as parameter and then use
 * the resulting values as if they were passed directly.
 *
 * @internal
 */
export interface IWithExecutionLoading<TProps> {
    /**
     * Specify export title that will be used unless the export function caller sends their own custom title.
     *
     * @param props - props to retrieve export title from
     */
    exportTitle: string | ((props: TProps) => string);
    /**
     * Specify a factory function to create data promises, based on props and optionally the data window size.
     *
     * This is where the data is actually being loaded. And the HOC hides the promise from the wrapped component
     * which just receives the data.
     *
     * @param props - wrapped component props
     * @param window - data view window to retrieve, not specified in case all data should be retrieved
     * @param signal - abort signal, will be used to cancel the execution
     */
    promiseFactory: (props: TProps, window?: DataViewWindow, signal?: AbortSignal) => Promise<DataViewFacade>;
    /**
     * Specify data view window to retrieve from backend.
     *
     * @remarks
     * If specified as function, the function can return undefined in case all data must be retrieved.
     */
    window?: DataViewWindow | ((props: TProps) => DataViewWindow | undefined);
    /**
     * Specify event callbacks which the HOC will trigger in different situations.
     */
    events?: IWithLoadingEvents<TProps> | ((props: TProps) => IWithLoadingEvents<TProps>);
    /**
     * Customize, whether execution & data loading should start as soon as component is mounted.
     *
     * Default is true. When not loading on mount, the wrapped component can trigger the load by calling the
     * reload() function which the HOC injects into its props.
     */
    loadOnMount?: boolean | ((props: TProps) => boolean);
    /**
     * Specify function that will be called during component prop updates and will be used to
     * determine whether execution should be re-run and data reloaded.
     *
     * @param prevProps - previous props
     * @param nextProps - next props
     */
    shouldRefetch?: (prevProps: TProps, nextProps: TProps) => boolean;
    /**
     * Optionally enable real execution cancellation.
     *
     * This means that if the execution request is not yet finished and the execution changes,
     * the request will be cancelled and the new execution will be started.
     *
     * Default: false
     */
    enableExecutionCancelling?: boolean | ((props: TProps) => boolean);
}
/**
 * A React HOC responsible for orchestrating resolution of a data promise (e.g. data to load).
 *
 * This component offers more flexibility in regards to how to obtain the data - all that is encapsulated
 * into a promise of data. For most use cases, the withExecution HOC is a better fit.
 *
 * @internal
 */
export declare function withExecutionLoading<TProps>(params: IWithExecutionLoading<TProps>): (WrappedComponent: React.ComponentType<TProps & WithLoadingResult>) => React.ComponentClass<TProps>;
//# sourceMappingURL=withExecutionLoading.d.ts.map